import edu.wpi.rail.jrosbridge.*;
import edu.wpi.rail.jrosbridge.callback.TopicCallback;
import edu.wpi.rail.jrosbridge.messages.*;
import edu.wpi.rail.jrosbridge.services.*;
import edu.wpi.rail.*;
import java.util.concurrent.TimeUnit;

public class main {
	
	
	public Ros createRos(String connectTo){
		Ros ros = new Ros(connectTo);
		return ros;
	}
	
	public Topic createTopic(Ros ros,String name, String type){
		Topic topic_name = new Topic(ros, name, type);
		return topic_name;
	}
	
	public Message createMessage(String msg){
		Message message_name = new Message(msg);
		return message_name;
	}
	
	public Service createService(Ros ros,String name, String type){
		Service service_name=new Service(ros, name, type);
		return service_name;
	}
	
	public ServiceRequest createServiceRequest(String request){
		ServiceRequest request_name = new ServiceRequest(request);
		return request_name;
	}
	
	public ServiceResponse createServiceResponse(Service service,ServiceRequest request){
		ServiceResponse response_name = service.callServiceAndWait(request);
		return response_name;
	}
	
	/**
	 * Main Method testing things:
	 * @param args
	 * @throws InterruptedException
	 */
	public static void main(String[] args) throws InterruptedException {
		//DO: (In Terminal:)     roslaunch rosbridge_server rosbridge_websocket.launch  (source this terminal!) 
		//DO: (In new Terminal:) rostopic echo /echo
		//DO: (In new Terminal:) rostopic pub /echo_back "std_msgs/String" "hi there"
		//DO: (In new Terminal:) rosrun beginner_tutorials add_two_ints_server (see Tut:http://wiki.ros.org/ROS/Tutorials/ExaminingServiceClient)
		//DO: (Testing in a Terminal:) java -jar test.jar
		
		Ros ros = new Ros("localhost");
     	ros.connect();

     	//1. [TOPICS] Send Messages from JAVA to ROS
		Topic echo = new Topic(ros, "/echo", "std_msgs/String");
		Message toSend1 = new Message("{\"data\": \"hello, world!\"}");
		Message toSend2 = new Message("{\"data\": \"hello, again!\"}");
		echo.publish(toSend1);
		echo.publish(toSend2);	
		System.out.println("Send data to ros   topic:"+echo.getName()+"  ThrottleRate: "+echo.getThrottleRate()+" Type: "+echo.getType());
		System.out.println(" Class: "+echo.getClass()+"  Compression: "+echo.getCompression()+" Ros: "+echo.getRos());
		
		//2. [TOPICS] Receive Messages send from ROS to JAVA
		Topic echoBack = new Topic(ros, "/echo_back", "std_msgs/String");
		System.out.println("Try to subscribe a Topic: ");
		echoBack.subscribe(new TopicCallback() {
			public void handleMessage(Message message) {
				System.out.println("From ROS: " + message.toString());
			}
		});
		//echoBack.unsubscribe(); //if you like.
		
		//3. [SERVICES] Start a Client in JAVA that uses a Service Server running in ROS
		//The following is similar to: rosrun beginner_tutorials add_two_ints_client 10 20
		System.out.println("Run a service: ");
		Service addTwoInts = new Service(ros, "/add_two_ints", "beginner_tutorials/AddTwoInts");

		ServiceRequest request = new ServiceRequest("{\"a\": 10, \"b\": 20}");
		ServiceResponse response = addTwoInts.callServiceAndWait(request);
		System.out.println(response.toString());
		
		//Sleep 2 secs before disconnect
		TimeUnit.SECONDS.sleep(2);
		ros.disconnect();
	}
}

/**Hints:
 * 1. The following WARN Message in the terminal appears:
 * [WARN] [1502881399.078792]: Could not process inbound connection: [/rosbridge_websocket] is 
 * not a publisher of [/echo]. Topics are [['/rosout', 'rosgraph_msgs/Log']]{'message_definition': 
 * 'string data\n', 'callerid': '/rostopic_8491_1502881362133', 'tcp_nodelay': '0', 
 * 'md5sum': '992ce8a1687cec8c8bd883ec73ca41d1', 'topic': '/echo', 'type': 'std_msgs/String'}
 * 
 * If this message comes up:
 * [ERROR] [1504248880.320547]: [Client 0] [id: call_service:/add_two_ints:4] call_service InvalidPackageException: Unable to load the manifest for package beginner_tutorials. Caused by: beginner_tutorials
 * Source the beginner_tutorials package first! in the terminal you run the rosbridge suite.
 * 
 * 2. Services:
 * Request / reply is done via a Service, which is defined by a pair of messages: 
	one for the request and one for the reply. A providing ROS node offers a 
	service under a string name, and a client calls the service by sending the 
	request message and awaiting the reply. Client libraries usually present this 
	interaction to the programmer as if it were a remote procedure call.
	Services are defined using srv files, which are compiled into source code by a ROS client library. 
	
	Implementation Class missing: use tyrus stand alone client 1.9.jar instead!
	Exception in thread "main" java.lang.RuntimeException: Could not find an implementation class.
	at javax.websocket.ContainerProvider.getWebSocketContainer(ContainerProvider.java:73)
	at jrosbridge.Ros.connect(Ros.java:203)
	at main.main(main.java:54)
 * */

